@@ -1,35 +1,71 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2010 Infrae. All rights reserved.
+# Copyright (c) 2010-2012 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id$
 
-import operator
 import functools
-import codecs
 import lxml.etree
 
 from infrae.testbrowser.interfaces import IFormControl, IForm
-from infrae.testbrowser.expressions import ExpressionResult
-from infrae.testbrowser.utils import File, resolve_url
+from infrae.testbrowser.interfaces import IClickableFormControl
+from infrae.testbrowser.interfaces import ISubmitableFormControl
+from infrae.testbrowser.utils import File, ResultSet, resolve_url
+from infrae.testbrowser.utils import parse_charset, charset_encoder
 
 from zope.interface import implements
 
 
+class ControlResultSet(ResultSet):
+
+    def __init__(self, controls, name):
+
+        def prepare(control):
+            key = getattr(control, name, 'missing')
+            return (key.lower(), key, control)
+
+        super(ControlResultSet, self).__init__(map(prepare, controls))
+
+
+class ControlExpressions(object):
+
+    def __init__(self, form):
+        self.__form = form
+        self.__expressions = {}
+
+    def add(self, name, expression):
+        self.__expressions[name] = expression
+
+    def __getattr__(self, name):
+        if name in self.__expressions:
+            expression = self.__expressions[name]
+
+            def matcher(control):
+                for key, value in expression[0].items():
+                    if getattr(control, key, None) != value:
+                        return False
+                return True
+
+            return ControlResultSet(
+                filter(matcher, self.__form.controls.values()),
+                expression[1])
+        raise AttributeError(name)
+
+
 class Control(object):
     implements(IFormControl)
 
-    def __init__(self, form, html):
+    def __init__(self, form, element_type, element, register_submit):
         self.form = form
-        self.html = html
-        self.__name = html.get('name')
+        self.html = element
+        self.__name = element.get('name')
         assert self.__name is not None
         self.__multiple = False
-        if html.tag == 'select':
-            self.__type = 'select'
-            self.__multiple = html.get('multiple', False) is not False
+        self.__type = element_type
+        submit = None
+        if element_type == 'select':
+            self.__multiple = element.get('multiple', False) is not False
             self.__value = [] if self.__multiple else ''
             self.__options = []
-            for option in html.xpath('descendant::option'):
+            for option in element.xpath('descendant::option'):
                 value = option.get('value', None)
                 if value is None:
                     value = option.text_content()
@@ -39,17 +75,49 @@ class Control(object):
                         self.__value = value
                     else:
                         self.__value.append(value)
+            if not self.__value and not self.__multiple and self.__options:
+                # In case of non multiple select, the first value
+                # should be selected by default
+                self.__value = self.__options[0]
+
+            def submit(encoder):
+                if self.__multiple:
+                    return [(self.name, encoder(value)) for value in self.value]
+                return [(self.name, encoder(self.value))]
+
         else:
-            if html.tag == 'textarea':
-                self.__type = 'textarea'
-                self.__value = html.text_content()
+            if element_type == 'textarea':
+                self.__value = element.text_content()
             else:
-                self.__type = html.get('type', 'submit')
-                self.__value = html.get('value', '')
+                self.__value = element.get('value', '')
             self.__options = []
+
+            if element_type == 'file':
+                submit = lambda encoder: [(self.name, File(self.value))]
+            else:
+
+                def submit(encoder):
+                    if self.__multiple:
+                        if len(self.__value):
+                            return [(self.name, encoder(self.__value[0]))]
+                        return []
+                    return [(self.name, encoder(self.__value))]
+
         self.__checked = False
-        if self.checkable:
-            self.__checked = html.get('checked', False) is not False
+        if element_type in ['radio', 'checkbox']:
+            self.__checked = element.get('checked', False) is not False
+
+            def submit(encoder):
+                if self.__multiple:
+                    return [(self.name, encoder(value)) for value in self.value]
+                if self.checkable and not self.__checked:
+                    return []
+                if not self.__value:
+                    return [(self.name, 'checked')]
+                return [(self.name, encoder(self.__value))]
+
+        if submit is not None:
+            register_submit(submit)
 
     @apply
     def name():
@@ -76,9 +144,12 @@ class Control(object):
             if self.multiple:
                 if not isinstance(value, list) or isinstance(value, tuple):
                     value = [value]
-                for subvalue in value:
-                    if subvalue not in self.__options:
-                        raise AssertionError(u"Invalid choice %s" % subvalue)
+                if self.__type in ['radio', 'checkbox', 'select']:
+                    for subvalue in value:
+                        assert subvalue in self.__options, \
+                            u"Invalid choice %s" % subvalue
+                else:
+                    assert len(value) == len(self.__value), u"Not enough values"
             else:
                 if isinstance(value, int):
                     value = str(value)
@@ -119,16 +190,37 @@ class Control(object):
             self.__checked = bool(value)
         return property(getter, setter)
 
-    def _extend(self, html):
-        assert self.__type == html.get('type', 'submit'), \
-            u'%s: control extended with a control type' % html.name
-        if self.__type == 'submit':
+    def _extend(self, element_type, element, register_submit):
+        if self.__type == 'submit' and element_type == 'submit':
             # We authorize to have more than one submit with the same name
             return
-        assert self.__type in ['checkbox', 'radio'], \
-            u'%s: only checkbox and radio can be multiple inputs' % html.name
-        assert self.name  == html.name, \
-            u'%s: control extended with a different input' % html.name
+        if self.__type not in ['checkbox', 'radio']:
+            # Support for multiple fields (hidden, other)
+            assert element_type not in ['file', 'submit', 'select'], \
+                u"%s: multiple input or mixing input %s and %s is not supported" % (
+                element.name, self.__type, element_type)
+            if self.__type != element_type:
+                self.__type = 'mixed'
+            if not self.__multiple:
+                self.__multiple = True
+                self.__value = [self.__value]
+            value_index = len(self.__value)
+            if element_type == 'textarea':
+                self.__value.append(element.text_content())
+            else:
+                self.__value.append(element.get('value', ''))
+
+            def submit(encoder):
+                if len(self.__value) > value_index:
+                    return [(self.name, encoder(self.__value[value_index]))]
+                return []
+
+            register_submit(submit)
+            return
+        # Checkbox, radio
+        assert self.__type == element_type, \
+            u'%s: control extended with a different control type (%s with %s)' % (
+            element.name, self.__type, element_type)
         if not self.options:
             # Firt time the control is extended
             self.html = [self.html]
@@ -144,8 +236,8 @@ class Control(object):
                     self.__value.append(value)
                 else:
                     self.__value = value
-        value = html.get('value', '')
-        if html.get('checked', False) is not False:
+        value = element.get('value', '')
+        if element.get('checked', False) is not False:
             if self.__multiple:
                 self.__value.append(value)
             else:
@@ -153,19 +245,7 @@ class Control(object):
                     u'Not multiple control with multiple value'
                 self.__value = value
         self.__options.append(value)
-        self.html.append(html)
-
-    def _submit_data(self, encoder):
-        if self.checkable:
-            if not self.checked:
-                return []
-            if not self.value:
-                return [(self.name, 'checked')]
-        elif self.multiple:
-            return [(self.name, encoder(value)) for value in self.value]
-        if self.type in ['file']:
-            return [(self.name, File(self.value))]
-        return [(self.name, encoder(self.value))]
+        self.html.append(element)
 
     def __str__(self):
         if isinstance(self.html, list):
@@ -177,95 +257,44 @@ class Control(object):
 
 
 class ButtonControl(Control):
+    implements(IClickableFormControl)
 
-    def __init__(self, form, html):
-        super(ButtonControl, self).__init__(form, html)
+    def click(self):
+        pass
+
+
+class SubmitControl(Control):
+    implements(ISubmitableFormControl)
+
+    def __init__(self, form, element_type, element, register_submit):
+        super(SubmitControl, self).__init__(
+            form, element_type, element, lambda r: None)
         self.__selected = False
 
+        def submit(encoder):
+            if self.__selected:
+                return [(self.name, encoder(self.value))]
+            return []
+
+        register_submit(submit)
+
     def submit(self):
         self.__selected = True
         return self.form.submit()
 
     click = submit
 
-    def _submit_data(self, encoder):
-        if not self.__selected:
-            return []
-        return [(self.name, encoder(self.value))]
-
 
 FORM_ELEMENT_IMPLEMENTATION = {
-    'submit': ButtonControl,
-    'image': ButtonControl}
+    'button': ButtonControl,
+    'submit': SubmitControl,
+    'image': SubmitControl
+}
 
-
-def parse_charset(charsets):
-    """Parse form accept charset and return a list of charset that can
-    be used in Python.
-    """
-    seen = set()
-
-    def resolve_charset(charset):
-        if not charset:
-            return None
-        try:
-            name = codecs.lookup(charset).name
-            if name in seen:
-                return None
-            seen.add(name)
-            return name
-        except LookupError:
-            return None
-        return None
-
-    return filter(lambda c: c != None,
-                  map(resolve_charset,
-                      reduce(operator.add,
-                             map(lambda c: c.split(), charsets.split(',')))))
-
-
-def charset_encoder(charset, value):
-    """Encoder a value in the given charset.
-    """
-    if isinstance(value, unicode):
-        return value.encode(charset, 'ignore')
-    return str(value)
-
-
-class Controls(ExpressionResult):
-
-    def __init__(self, controls, name):
-
-        def prepare(control):
-            key = getattr(control, name, 'missing')
-            return (key.lower(), key, control)
-
-        super(Controls, self).__init__(map(prepare, controls))
-
-
-class ControlExpressions(object):
-
-    def __init__(self, form):
-        self.__form = form
-        self.__expressions = {}
-
-    def add(self, name, expression):
-        self.__expressions[name] = expression
-
-    def __getattr__(self, name):
-        if name in self.__expressions:
-            expression = self.__expressions[name]
-
-            def matcher(control):
-                for key, value in expression[0].items():
-                    if getattr(control, key, None) != value:
-                        return False
-                return True
-
-            return Controls(
-                filter(matcher, self.__form.controls.values()),
-                expression[1])
-        raise AttributeError(name)
+FORM_ELEMENT_DEFAULT = {
+    'button': 'submit',
+    'input': 'text'
+}
 
 
 class Form(object):
@@ -287,55 +316,37 @@ class Form(object):
         self.inspect.add('actions', ({'type': 'submit'}, 'value'))
         self.__browser = browser
         self.__control_names = []
+        self.__control_submits = []
         self.__populate_controls()
 
     def __populate_controls(self):
         __traceback_info__ = 'Error while parsing form: \n\n%s\n\n' % str(self)
         # Input tags
-        for input_node in self.html.xpath('descendant::input'):
+        # XXX: Test that the order is conserved form the source page.
+        # XXX: Use an ordered dict ?
+        for input_node in self.html.xpath(
+            'descendant::input|descendant::select|descendant::textarea|' \
+                'descendant::button'):
+
             input_name = input_node.get('name', None)
             if not input_name:
                 # No name, not a concern to this form
+                # XXX: Should default to the good default
                 continue
+            if input_node.tag in ['input', 'button']:
+                input_type = input_node.get(
+                    'type', FORM_ELEMENT_DEFAULT[input_node.tag])
+            else:
+                input_type = input_node.tag
             if input_name in self.controls:
-                self.controls[input_name]._extend(input_node)
+                self.controls[input_name]._extend(
+                    input_type, input_node, self.__control_submits.append)
             else:
-                input_type = input_node.get('type', 'submit')
                 factory = FORM_ELEMENT_IMPLEMENTATION.get(input_type, Control)
-                self.controls[input_name] = factory(self, input_node)
+                self.controls[input_name] = factory(
+                    self, input_type, input_node, self.__control_submits.append)
                 self.__control_names.append(input_name)
 
-        # Select tags
-        for select_node in self.html.xpath('descendant::select'):
-            select_name = select_node.get('name', None)
-            if not select_name:
-                # No name, not a concern
-                continue
-            assert select_name not in self.controls
-            self.controls[select_name] = Control(self, select_node)
-            self.__control_names.append(select_name)
-
-        # Textarea tags
-        for text_node in self.html.xpath('descendant::textarea'):
-            text_name = text_node.get('name', None)
-            if not text_name:
-                # No name, not a concern
-                continue
-            assert text_name not in self.controls
-            self.controls[text_name] = Control(self, text_node)
-            self.__control_names.append(text_name)
-
-        # Button tags
-        for button_node in self.html.xpath('descendant::button'):
-            button_name = button_node.get('name', None)
-            if not button_name:
-                # No name, not a concern
-                continue
-            assert button_name not in self.controls, \
-                u'Duplicate input %s in form %s' % (button_name, self.name)
-            self.controls[button_name] = ButtonControl(self, button_node)
-            self.__control_names.append(button_name)
-
     def get_control(self, name):
         if name not in self.controls:
             raise AssertionError(u'No control %s' % name)
@@ -348,12 +359,17 @@ class Form(object):
             if value is None:
                 value = self.controls[name].value
             form.append((name, value))
-        for name in self.__control_names:
-            control = self.controls[name]
-            form.extend(control._submit_data(encoder))
+        for submitter in self.__control_submits:
+            form.extend(submitter(encoder))
         return self.__browser.open(
-            self.action, method=self.method,
-            form=form, form_enctype=self.enctype)
+            self.action,
+            method=self.method,
+            form=form,
+            form_charset=self.accept_charset[0],
+            form_enctype=self.enctype)
 
     def __str__(self):
         return lxml.etree.tostring(self.html, pretty_print=True)
+
+    def __repr__(self):
+        return '<form action="%s" />' % (self.action)
